using System;
using System.Runtime.InteropServices; /* Provides mappings between C# and native code */


/* This example calls either a Windows or Linux version of the NAG Library based on
 * which OS the example is being executed on*/

namespace NumericalAlgorithmsGroup
{
  class Quadrature
    {
      static int kount;

        /* NAG routine declarations copied from flcsdnet64.cs & flcsdnet64_linux.cs*/

        /* Both Windows and Linux routine declarations included as the dll is not 
         * loaded unless the routine is called explicitly.
         * The Linux declaration has an underscore appended to differentiate it*/

        /* Delegate object, note that the scalar parameter is 'ref' */
        public delegate double D01AJF_F_DELEGATE(
                                               ref double X
                                               );

      [DllImport("NLW6I271E_nag.dll")] /* Import from DLL, the C# compiler provides a rudimentary check of the signature */
      public static extern void D01AJF(
                                       D01AJF_F_DELEGATE F,
                                       ref double A,
                                       ref double B,
                                       ref double EPSABS,
                                       ref double EPSREL,
                                       ref double RESULT,
                                       ref double ABSERR,
                                       [Out] double[] W,
                                       ref int LW,
                                       [Out] int[] IW,
                                       ref int LIW,
                                       ref int IFAIL
                                       );

      [DllImport("libnag_nag.so", EntryPoint="d01ajf_")] 
      public static extern void D01AJF_(
                                       D01AJF_F_DELEGATE F,
                                       ref double A,
                                       ref double B,
                                       ref double EPSABS,
                                       ref double EPSREL,
                                       ref double RESULT,
                                       ref double ABSERR,
                                       [Out] double[] W,
                                       ref int LW,
                                       [Out] int[] IW,
                                       ref int LIW,
                                       ref int IFAIL
                                       );

      public static void Main()
      {
          double epsrel = 1.0e-4;
          double epsabs = 0.0;
          int lw = 2000;
          double [] w = new double[lw];
          int liw = lw/4;
          int [] iw = new int [liw];
          int ifail = 1;
          double abserr = 0.0;
          double result=0;
          /* Create a delegate object */
          D01AJF_F_DELEGATE FUNC = new D01AJF_F_DELEGATE(func);
          double a = 0.0;
          double b = 2.0*Math.PI;
          if (RuntimeInformation.IsOSPlatform(OSPlatform.Windows))
            {
              D01AJF(FUNC, ref a, ref b, ref epsabs, ref epsrel, ref result,
              ref abserr, w, ref lw, iw, ref liw, ref ifail);
            }
          else if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
            {
              D01AJF_(FUNC, ref a, ref b, ref epsabs, ref epsrel, ref result,
              ref abserr, w, ref lw, iw, ref liw, ref ifail);
            }
          else
            {
              Console.WriteLine("Unsupported OS Platform");
              return;
            }
          if (ifail != 0)
            {
                Console.WriteLine("ifail = {0}", ifail);
              // throw exception
            }
          else
            {
              Console.WriteLine("Approximation to the integral = {0, 9:f5}", result);
              Console.WriteLine("Estimate of the absolute error = {0, 9:e2}", abserr);
              Console.WriteLine("Number of function evaluations ={0}", kount);
              Console.WriteLine("Number of subintervals used = {0}", iw[0]);
            }
      }
      public static double func(ref double x)
      {
          ++kount;
          double pi = Math.PI;
          double retval;
          retval =  (x*Math.Sin(x*30.0)/Math.Sqrt(1.0-x*x/(pi*pi*4.0)));
          return retval;
      }
    }
}
